import pandas as pd
import numpy as np


class SyntheticDataGenerator:
    def __init__(self, loa, max_month_id=852, max_priogrid_id=60000, max_country_id=242):
        self.loa = loa
        self.max_month_id = max_month_id
        self.max_priogrid_id = max_priogrid_id
        self.max_country_id = max_country_id
        self.df = None

    def generate_dataframe(self):
        # Initialize data
        data = {
            'ln_ged_sb_dep': [],
            'ged_sb': [],
            'ged_os': [],
            'ged_ns': [],
            'ln_pop_gpw_sum': [],
            'decay_ged_sb_5': [],
            'decay_ged_sb_25': [],
            'decay_ged_sb_100': [],
            'decay_ged_sb_500': [],
            'decay_ged_os_5': [],
            'decay_ged_os_25': [],
            'decay_ged_os_100': [],
            'decay_ged_os_500': [],
            'decay_ged_ns_5': [],
            'decay_ged_ns_25': [],
            'decay_ged_ns_100': [],
            'decay_ged_ns_500': [],
            'splag_1_1_sb_1': [],
            'splag_1_decay_ged_sb_1': []
        }

        # Create the multi-index and populate data
        if self.loa == 'cm':
            country_ids = range(1, self.max_country_id + 1)
            index = pd.MultiIndex.from_product([range(1, self.max_month_id + 1), country_ids], names=('month_id', 'country_id'))
            n = len(country_ids) * self.max_month_id

        else:  # Assumes 'pgm'
            priogrid_ids = range(1, self.max_priogrid_id + 1)
            index = pd.MultiIndex.from_product([range(1, self.max_month_id + 1), priogrid_ids], names=('month_id', 'priogrid_id'))
            n = len(priogrid_ids) * self.max_month_id

        for col in data.keys():
            data[col] = np.random.rand(n)

        # Create DataFrame
        self.df = pd.DataFrame(data, index=index)
        return self.df

    def generate_csv(self, filename):
        self.df.to_csv(filename)
        return self.df

    def generate_parquet(self, filename):
        self.df.to_parquet(filename)
        return self.df
